Attribute VB_Name = "Module1"

''*******************************
''Copywrite Johns Hopkins University ECE Dept - R. Jenkins
''Functions for Automatic USB Operations compatible with XSA/XST board.
''*******************************
'This module contains functions:
'setmode()                  - to send a feature mode to the FPGA for project control
'Load_MemUSB, Read_MemUSB   - to load and read DRAM with options set by select buttons
'SendTones, SendSweep, DDS_on, DDS_off, and Delay - to support DDS tones testing

'public variables and arrays to support USB API calls
Public bAlertable As Long
Public Capabilities As HIDP_CAPS
Public DataString As String
Public DetailData As Long
Public DetailDataBuffer() As Byte
Public DeviceAttributes As HIDD_ATTRIBUTES
Public DevicePathName As String
Public DeviceInfoSet As Long
Public ErrorString As String
Public EventObject As Long
Public HIDHandle As Long
Public HIDOverlapped As OVERLAPPED
Public LastDevice As Boolean
Public MyDeviceDetected As Boolean
Public MyDeviceInfoData As SP_DEVINFO_DATA
Public MyDeviceInterfaceDetailData As SP_DEVICE_INTERFACE_DETAIL_DATA
Public MyDeviceInterfaceData As SP_DEVICE_INTERFACE_DATA
Public Needed As Long
'Public OutputReportData(7) As Byte
Public PreparsedData As Long
Public ReadHandle As Long
Public result As Long
Public Security As SECURITY_ATTRIBUTES
Public Timeout As Boolean
Public manuf As String

'We maintain the read and send buffers as global static, so that changing any given byte
'  while preserving the other byte values sent to the FPGA is simple. Just change that
'  byte and re-send the report. Byte 0 of these arrays is the report id and is always 0.
Public ReadBuffer() As Byte 'These used by writeReport and ReadReport for data xfers
Public SendBuffer() As Byte '

Public FPGAHandle As Long
Public Err_flag As Integer, pktct As Long
Public delay_done As Integer  'flag for Delay Timer
Public NumInBufs As Integer
'Mem Burst Read parameter
Public IMem As Long, Imax As Long, accum As String
'--------------------------------------------

Public Sub SetMode(mode As Byte)
'use a set_feature call to send the mode byte to FPGA
'We use the same static SendBuffer as Set_report
SendBuffer(0) = 0
SendBuffer(1) = mode
SNDPKT:
result = HidD_SetFeature _
   (HIDHandle, _
    SendBuffer(0), _
    CLng(Capabilities.FeatureReportByteLength))
If result = 0 Then
    'pktct = pktct + 1
    'frmMain.DebugOut.RemoveItem (frmMain.DebugOut.ListCount - 1) 'Rewrite last line in list box.
    'frmMain.DebugOut.AddItem "*** Packet Retries= " & pktct
    'frmMain.DebugOut.ListIndex = frmMain.DebugOut.ListCount - 1
    'GoTo SNDPKT     'try again
    Call frmMain.DisplayResultOfAPICall("HidD_SetFeature")
    Exit Sub
End If
End Sub

''==========================================================================
'**********************************************************************
'* Function to load FPGA dram, either from a file or a coded algorithm below.
'* This works with FPGA state machine that receives and stores 8-byte packets,
'* sent by subroutine "frmMain.WriteReport". Each byte pair in the packet specifies
'* a 16-bit value, first high-byte, then low-byte for each FPGA write into mem.
'* If only byte loads are desired, you can change code to send a zero upper byte.
'* If the resident FPGA state machine is only loading a single byte per memory loc,
'* the upper byte will just be ignored.
'**********************************************************************
Public Sub Load_MemUSB()
'Assumes XSA board is reset and in GETBYTES state, waiting for first packet.
'50mhz state machine treats each packet as four 16-bit words to be stored sequentially.
'Then it waits for the new_dat flag to go low before returning to GETBYTES state.

Dim MDat As Long
Dim J As Long, K As Long, L As Long
Dim V As Byte, FileN As String, hexline As String, accum As String
Dim I As Double, Freq As Double, Imax As Double 'these are flt for the sine wave case
'----------------------
pktct = 0   '***DEBUG - reset the displayed pkt error count
frmMain.DebugOut.RemoveItem (frmMain.DebugOut.ListCount - 1) 'Rewrite last line in list box.
frmMain.DebugOut.AddItem "***Write-Packet Errors= " & pktct
frmMain.DebugOut.ListIndex = frmMain.DebugOut.ListCount - 1
'-----------------------
If frmMain.LdOption1.Value = True Then  'This option says load from a coded algorithm
    num = InputBox("Enter number of location to write. " & vbLf & _
        "Negative number means that many locs will be zeroed.", _
        "Size of DRAM Load")
    If (Len(num) = 0) Or (num = vbCancel) Then Exit Sub
    Imax = CDbl(Abs(num)) + 4#    'Num rounded up by 4.
End If
'-----------------------
If frmMain.LdOption3.Value = True Then   'This option says load a DDS sine wave
    num = InputBox("Enter number of bits in sine wave look-up address. " & vbLf & _
        "16-bit 2's compliment values will be loaded for the sine wave samples.", _
        "Size of Sine-Wave Look-up Addresses")
    If (Len(num) = 0) Or (num = vbCancel) Then Exit Sub
    Imax = 2 ^ num + 4#       'this will always be a multiple of 4
    Freq = (2# * 3.1415926535898) / (2 ^ num) 'Freq set for one cycle in 2^N samples.
End If
'----------------------
R = MsgBox("Ready to load RAM. Num = " & Imax & " Locations.   Continue?", vbOKCancel)
If R = vbCancel Then Exit Sub
'Ready to start - set FPGA mode to 1 so the mem access state machine goes to write state
Call SetMode(1)
'--------FPGA should now be in getbytes state waiting for first data packet.

'Jump to algorithm loop if no file is being used ---------------->
If frmMain.LdOption2 = False Then GoTo NOFILE 'otherwise continue on below

'======In this section we load sequentially from a hex file ===========
' The load option is set to load hex file line by line.
'*****currently set up for single byte data only. Upper byte is set to 0.
FileN = "C:\LoadVals.txt"
Close #1
R = MsgBox("Ready to load RAM from hex file  " & FileN & ".   Continue?", vbOKCancel)
If R = vbCancel Then Exit Sub

Open FileN For Input As 1   ' Open source file to read line by line
K = 0: L = 0        ' K is packet pointer, L is line number
Lineloop:  If EOF(1) Then GoTo DONE 'get the next line or exit
Line Input #1, hexline      'read  and count next line
L = L + 1:
c = InStr(hexline, ";")    'look for a comment start, and trim it away if there
If c > 0 Then hexline = Mid(hexline, 1, c - 1)
TABS: c = InStr(hexline, vbTab)    'look for tabs and replace with space
If c > 0 Then
      Mid(hexline, c, 1) = " "
      GoTo TABS
End If
hexline = Trim(hexline)    'trim any leading or trailing blanks
If 0 = Len(hexline) Then GoTo Lineloop  'skip blank lines

Byteloop:
 hexline = LTrim(hexline) 'trim leading blanks
 If (Len(hexline) = 1) Then      'Cant end in a single hex digit.
    Close #1
    MsgBox "invalid line in file. line no. = " & L
    Exit Sub
 End If
 
MDat = Val("&H" & Left(hexline, 2))         'get the next hex byte on this line.
SendBuffer(K + 2) = CByte(MDat And &HFF)    'store Mdat into buffer low byte
SendBuffer(K + 1) = 0                       'zero out upper byte.
K = (K + 2) Mod 8                   'count byte pairs for a full packet
If K = 0 Then       'Write the SendBuffer to device every fourth value
    'Call frmMain.WriteReport   'WriteReport uses interrupts if EP1 defined
    Call frmMain.SendReport     'SendReport uses control xfers
    DoEvents
End If

hexline = Right(hexline, Len(hexline) - 2) 'drop the used characters.
If 0 = Len(hexline) Then GoTo Lineloop
GoTo Byteloop   'still chars left to process on this line.

DONE:
Close #1
Call SetMode(0)      'reset project mode back to 0
MsgBox "Load complete" & vbCrLf & CStr(L) & " lines processed."
Exit Sub            'FINISHED with file load

''====In this section we load a pattern generated by a coded program loop=====
NOFILE:         'Num = 0 -> load a pattern with program loop below
'--------------------------------------------
J = -1                 'in case it's needed below for generating a VGA pattern
K = 0: SendBuffer(0) = 0
'---------------------------------------------
For I = 0 To Imax - 1    'ILOOP to load values into dram by 16-bit words.

If frmMain.LdOption3.Value = True Then 'This option says load a DDS sine wave
    MDat = CLng(32760# * Sin(I * Freq))  '2's compl. sine wave with 15-bit addr, 16-bit samples
ElseIf num < 0 Then
    MDat = 0    'zero out the mem.
''Uncomment a line in this block for simple testing of mem load and read
Else: 'Mdat = (I Mod 254) + 256 * ((I + 1) Mod 254)
    MDat = I
    'Mdat = Asc("J") + 256 * Asc("R")    'fill with constant ascii characters

End If

GoTo SENDDAT        'comment this out if using an algorithm below---->
''--------------------------------------------------------------
 '--This block generates Vertical VGA stripes( 9-bit colors)----
'J = I Mod 8
 If (I Mod 16) = 0 Then J = (J + 1) Mod 8
 If J = 0 Then MDat = &H7       'blue
 If J = 2 Then MDat = &H38      'green
 If J = 1 Then MDat = &H1C0     'red
 If J = 3 Then MDat = &H1D8     'rgb=(6,3,0) ->orange
 If J = 4 Then MDat = &H147     '(5,0,7) ->purple
 If J = 6 Then MDat = &H1F8     '(7,7,0) ->cyan
 If J = 5 Then MDat = &H3F      '(0,7,7) ->yellow
 If J = 7 Then MDat = &H1C7     '(7,0,7) ->magenta
 If I > 200# * 256# Then
    MDat = &H49 * Fix(CDec(I Mod 256) / 32#)  'after 200 rows, make gray levels
 End If
''----Value set. Now send the bytes in packets-----
SENDDAT:
SendBuffer(K + 2) = CByte(MDat And &HFF)       'store Mdat low byte into buffer
SendBuffer(K + 1) = CByte(&HFF And Int(MDat / 256)) 'store upper byte into buffer.
K = (K + 2) Mod 8   'count byte pairs for a full packet
If K = 0 Then
    'Call frmMain.WriteReport   'Write SendBuf to FPGA every fourth value via EP1 intrrpt
    Call frmMain.SendReport  'Write SendBuffer to FPGA every fourth value via control EP0
    frmMain.Progress.Text = Int((100 * I) / Imax) & "%" 'update progress number.
    DoEvents
End If
'-------------------------------------------------
ILOOP: Next I   'keep on truckin' with the pattern being loaded

XIT:
SetMode (0)                 'reset back to mode 0 out of read dram mode=2
Err_flag = 0
'On Error GoTo 0             'Reset error handling
frmMain.LabelLocs.Caption = "Load Complete" & vbCrLf & CStr(I) & " Locations written."
Beep
End Sub



''=====================================================================================
'*************************************************************************
'* Memory 16-bit hex read for XSA boards using appropriate FPGA state machine.
'* Packets are filled by consecutive memory reads in the FPGA state machine.
'* The FPGA state machine pauses each packet until we do a read report to get data.
'* With 8-bit memory data, the high bytes are expected to be dummy 0 values.
'*************************************************************************
Public Sub Read_MemUSB()
'Read XSA DRAM with fpga state machine and write hex values or ascii values to
' C:/MemVals.txt, then open Notepad to view.(good font setting is Courier New-bold-11)
'FPGA returned packets have 4 16-bit values from consecutive dram locations.
'Dim MDat As Long, IMem As Long, Imax As Long
Dim K As Long, J As Long, MDat As Long, hvalue As String
Dim FileN, init As String    ', hvalue As String

'---***DEBUG----------------------------
pktct = 0   'reset the displayed missed-pkt count
frmMain.DebugOut.RemoveItem (frmMain.DebugOut.ListCount - 1) 'Rewrite last line in list box.
frmMain.DebugOut.AddItem "*** Packet Errors = " & pktct
frmMain.DebugOut.ListIndex = frmMain.DebugOut.ListCount - 1
'-------------------------------------

num = InputBox("Enter number of location to read. ", "READ DRAM")
If (Len(num) = 0) Or (num = vbCancel) Then GoTo XIT2
Imax = num
R = MsgBox("Ready to Read RAM.  Num = " & Imax & " Locations.  Continue?", vbOKCancel)
If R = vbCancel Then GoTo XIT2
FileN = "C:\MemVals.txt"    ' Open dump file and write the heading
Close #1                    'just in case it's left open
Open FileN For Output As 1

If frmMain.RdOption2.Value = True Then  'use different file format for ascii versus hex
    Print #1, "    Text Values Uploaded from RAM"
    Print #1, accum & "   -----------------------------" & vbCrLf
    accum = ""  'this is the output text accumulator.
Else
    Print #1, "       Hex Values Uploaded from RAM" & vbCrLf & "Hex Addr"
    accum = " 000000   "  'this is the output text row accumulator, starting at address 0.
End If

'We assume FPGA in RDPAUSE state with 1st packet loaded from dram, waiting for 1st read.

IMem = 0
'result = HidD_FlushQueue(HIDHandle) 'readbuffer only applies to interrupt transfers
'SetMode (34) 'send project mode = 2+32 to reset DRAM addr and start interrupt reads
SetMode (2) 'send project mode = 2 to reset DRAM addr and start control reads
'--======= Coment this block to run without Interrupt Timer =====================
' frmMain.TmrMemRd.Interval = 700     '300 millisec period for 64 packet burst.
' frmMain.TmrMemRd.Enabled = True
' Call frmMain.TmrMemRd_Timer
'Exit Sub
''----------------
'XIT2: SetMode (0)      'reset back to mode 0 out of read dram mode=2
'  Exit Sub
'End Sub

'Sub DUMMYRD()   'We preserve the old code body with a dummy Sub
''--============================================================================
'On Error GoTo ErrT
NEXT_IMEM:
If 0 = (IMem Mod 4) Then ''----Read next packet from FPGA------------
    K = 0   'reset the packet pointer
READUSB: '
    Call frmMain.ReadReport(1)  'read next packet via control xfer.
    'Call frmMain.ReadReport(3)   'rd next packet via interrupt. Doesn't solve pktct problem
    Call frmMain.SendReport     '**this cures pkt error problem but still dont know why.
    If Err_flag = 1 Then        'read packet error - try re-reading
        If pktct < 90 Then      'give up the ghost if too many
            GoTo READUSB
        Else:
            MsgBox "Halting - excessive packet errors. IMem=" & IMem
            GoTo XIT1
        End If
    End If
    DoEvents    'let text boxes on frame get updated
End If
''---------------------------
'Work through the read buffer in pairs for 16-bit values
If frmMain.RdOption2.Value = False Then GoTo HEXNUMS  'interpret as 16-bit hex nums=====>

'--- Process DRAM values as Ascii text ---------
'This option is specifically to support a PS2 keyboard lab project.
'Each 16-bit Mem value is interpreteded as two ascii characters
MDat = ReadBuffer(K + 1)    'high byte is 1st char
'If Mdat = 8 Then     'backspace one char unless accum is empty
'    accum = Left(accum, Len(accum) - 1)
'End If
If MDat = 13 Then      'This allows for <cr>'s in the stored text
    accum = accum & vbCrLf
Else: accum = accum & Chr(MDat)
End If
MDat = ReadBuffer(K + 2)    'low byte is 2nd char
If MDat = 13 Then      'Check for a <cr>
    accum = accum & vbCrLf
Else: accum = accum & Chr(MDat)
End If
GoTo ILOOP
'----Process dram values as 16-bit hex numbers--------
HEXNUMS:
MDat = ReadBuffer(K + 2) + ReadBuffer(K + 1) * 2 ^ 8  'combine bytes for 16-bit values
hvalue = "000" & Hex(MDat) & " " 'next 4 digit hex value, add leading zeros in case
accum = accum & Right(hvalue, 5)

LINE:   'write out current accumulated text line to file every 8 values.
If 7 = (IMem Mod 8) Then  '8 values/line, each St-machine cycle sends 8- or 16-bit value
    Print #1, accum     ' write it, and initialize accum for next line.
    accum = " " & Right("00000" & Hex(IMem + 1), 6) & "   "      'word count=addr
End If
'-------------------------------------------------
ILOOP:
K = (K + 2) Mod 8    'as we go, increment read-buffer pointer by byte pairs.
frmMain.Progress.Text = Int((100 * IMem) / Imax) & "%" 'update progress number.
If IMem = Imax Then GoTo XIT1
IMem = IMem + 1: GoTo NEXT_IMEM

XIT1:
  'TmrMemRd.Enabled = False
 'Give completion message.
  frmMain.LabelLocs.Caption = "Mem Read Ended." & vbCrLf & CStr(IMem) & " Locations read."
  Print #1, accum    'Write last results and close the text file.
  Close #1
  Shell "notepad c:\MemVals.txt", vbNormalFocus 'open the output text file with notepad
XIT2:  On Error GoTo 0             'Reset error handling
  SetMode (0)            'reset back to mode 0 out of read dram mode
  Exit Sub     '****************************
ErrT:        'Trap to print err info, or alternatively, ignore user Cancel or Xit from Open Dialog
  SetMode (0)
  MsgBox "Error: " & Err.Number & ", " & Err.Description & ", " & Err.source
  Exit Sub     '****************************
End Sub     '
'***************************************************************************

''=========================================================================
'' The Functions below support DDS projects and are not part of the USB test project
''=========================================================================

'*******************************************
Public Sub SendSweep() 'Send DDS control constants to synthesize acoustic tones
' Frequency sweep twice by sending hex DDS control values
'We assume the resident FPGA project is expecting on/off and DDS control bytes
'  in the first two bytes of the sent packet
Dim J As Long, V As String

DDS_on      'turn on sound, preserving other bytes in send buffer
For J = 0 To 3 * 24 - 1
    V = "&H" & Hex(1 + J Mod 24)
    'send the new note value
    SendBuffer(2) = Val(V)   'store note byte into static buffer
    Call frmMain.WriteReport   'Write the SendBuffer to fpga
    Delay (80) 'now kill some time to slow down sweep
NJ: Next J
Delay (200)
DDS_off    'turn the sound off after 200 ms
End Sub
'*******************************************
Public Sub SendTones(N As Integer)      'Send DDS constants to synthesize acoustic tones
'Old play a tune thru the codec via 50MHz DDS for XSA.
'We assume 32K 15-bit sine wave is stored in memory.
'We assume the resident FPGA project is expecting on/off and DDS control bytes
'  in the first two bytes of the sent packet
'DDS on/off uses PCOut(1), and PCOut(2) is the note control value.
'' "*"->.06 sec break, "$"->full note break, "-"->sustain note one beat(.250 sec)
''.06 sec break automatically inserted between back-to-back identical notes.
'*******************************************
Dim I As Long, J As Long, K As Long, M As Long
Dim L As String, V1 As String, V2 As String
Const L1 = "$13568acd-$dca86531"     'This does the C scale
Const L2 = "$668a6a83668a6-5-668aba8651356-*6"     'This does yankee doodle
Const L3 = "$ca8accc-aaa-cff-ca8accccaaca8"       'Mary had a little lamb

If N = 3 Then
    L = L3
ElseIf N = 2 Then
    L = L2
Else
    L = L1
End If

'Time delay is 250 msec for notes, 60 msec off break between identical notes.
V2 = "r"
For J = 1 To Len(L)
'DDS_on       'sound on - SendBuffer(1) controls on/off for DDS project
  V1 = Mid(L, J, 1)
  If V1 = "$" Then      'If a pause is scheduled, then
    DDS_off             'turn sound off for a pause.
    GoTo DONE           'Finished with this symbol
    End If
  If V2 = "$" Then  'Last symbol was a pause and this one isnt, then
    GoTo SEND       'send the new note and turn the codec back on.
    End If
  If V1 = V2 Then   'Back to back identical notes?
    DDS_off         'then turn sound off for 60 msec.
    Delay (60)
    GoTo SEND       'Now send the repeat note with codec back on
    End If
  If V1 = "-" Then
    GoTo DONE       'hold the last note for another beat.
    End If
  If V1 = "*" Then   'A scheduled break
    DDS_off         'sound off for 80 msec.
    Delay (80)
    DDS_on          'sound back on
    GoTo DONE
    End If
SEND:     'send the new note value
    SendBuffer(2) = Val("&H" & V1)   'store note byte into buffer(2)
    Call frmMain.WriteReport   'Write the SendBuffer to fpga
    DDS_on                     'sound on
DONE:  Delay (250) 'sustain each note for 250 msec, then get the next symbol
    V2 = V1         'save the current symbol
NJ: Next

Delay (400)           'hold final note for an extra 400 msec
DDS_off                     'Done. Turn the sound off

End Sub

Sub DDS_on()    'send the DDS on value in packet byte 1
'We assume the sendbuffer is static, holding the current note value
SendBuffer(1) = 1          'store dds "on" value into buffer
Call frmMain.WriteReport   'Write the SendBuffer to fpga
End Sub
Sub DDS_off()   'send the DDS 0ff value in packet byte 1
'We assume the sendbuffer is static, holding the current note value
SendBuffer(1) = 0          'store dds "off" value into buffer
Call frmMain.WriteReport   'Write the SendBuffer to fpga
End Sub

Sub Delay(dt As Long)
'Generate a delay using the TmrDelay timer control. The control is enabled here,
'  disables itself the first time it is called, then sets the delay_done flag high.
'This creates a one-shot delay of value dtmsec.
' 55 msec is about the best time resolution achievable
delay_done = 0
frmMain.TmrDelay.Interval = dt     'dt=1000 ->1 sec
frmMain.TmrDelay.Enabled = True    'enable the delay timer and wait for it to set flag
Do Until delay_done = 1
    'Beep
    DoEvents
Loop

End Sub

Sub test_Delay()
onoff:
    DDS_off       'then turn codec off for 1sec.
    Delay (1000)
    frmMain.Progress.Text = 1
    DoEvents
    DDS_on        'codec back on for 1 sec
    Delay (1000)
    frmMain.Progress.Text = 0
    DoEvents
GoTo onoff
End Sub
''===================end of DDS support===================================

